@@ -14,7 +14,7 @@
 	,"DEBPKG:debian/libnet_config_path - Set location of libnet.cfg to /etc/perl/Net as /usr may not be writable."
 	,"DEBPKG:debian/perlivp - https://bugs.debian.org/510895 Make perlivp skip include directories in /usr/local"
 	,"DEBPKG:debian/squelch-locale-warnings - https://bugs.debian.org/508764 Squelch locale warnings in Debian package maintainer scripts"
-	,"DEBPKG:debian/patchlevel - https://bugs.debian.org/567489 List packaged patches for 5.34.0-3ubuntu1.1 in patchlevel.h"
+	,"DEBPKG:debian/patchlevel - https://bugs.debian.org/567489 List packaged patches for 5.34.0-5ubuntu1.1 in patchlevel.h"
 	,"DEBPKG:fixes/document_makemaker_ccflags - https://bugs.debian.org/628522 [rt.cpan.org #68613] Document that CCFLAGS should include $Config{ccflags}"
 	,"DEBPKG:debian/find_html2text - https://bugs.debian.org/640479 Configure CPAN::Distribution with correct name of html2text"
 	,"DEBPKG:debian/perl5db-x-terminal-emulator.patch - https://bugs.debian.org/668490 Invoke x-terminal-emulator rather than xterm in perl5db.pl"
@@ -47,6 +47,9 @@
 	,"DEBPKG:fixes/x32-io-msg-skip - https://bugs.debian.org/922609 Skip io/msg.t on x32 due to broken System V message queues"
 	,"DEBPKG:fixes/encode-CVE-2021-36770 - mitigate @INC pollution when loading ConfigLocal"
 	,"DEBPKG:fixes/gdbm-1.20 - https://bugs.debian.org/993514 [GH #18915] Fix GDBM_File to compile with version 1.20 and earlier"
+	,"DEBPKG:fixes/gcc-braces-1 - [7169efc] https://bugs.debian.org/1009149 skip using gcc brace groups for STMT_START/END"
+	,"DEBPKG:fixes/gcc-braces-2 - [e08ee3c] https://bugs.debian.org/1009149 Partially Revert \"skip using gcc brace groups for STMT_START/END\""
+	,"DEBPKG:fixes/perl5db-cmd-l - https://bugs.debian.org/1010866 [b7a355b] lib/perl5db.pl: cmd_l() no longer exists."
 	,"DEBPKG:disable-libperl-tests -"
 	,"DEBPKG:CVE-2020-16156-1.patch - [PATCH] bugfix: signature verification type CANNOT_VERIFY was not recognized"
 	,"DEBPKG:CVE-2020-16156-2.patch - [PATCH] Add two new failure modes based on cpan_path"
